# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

# Name:           jsAlign.rb 1.5
# Author:         Jan Sandstrom   www.pixero.com
# Date:           23 June 2009
# Changes:	Added functionality to also work on groups/components.
# Description:		Use for aligning edges or groups/components.
#                		If using custom value with groups also use Align Group's: Max, Min or Center.


require 'sketchup.rb'


class Sketchup::Selection 
def vertices 
	selverts = []; 
	self.each {|e| 
		selverts.push e.vertices if (e.respond_to?(:vertices)); 
	} 
	return selverts.flatten.uniq; 
end ;  # method vertices
end ;  # class Sketchup::Selection 




def jsAlign 
	model = Sketchup.active_model
	entities = model.active_entities
	sel = model.selection
	
	if sel.empty?
     		UI.beep
     		UI.messagebox("Nothing Selected!")
     		return nil
  	end#if
  	
  	### Get all edges in selection
	
	edges=[]
	for e in sel
  		if e.typename=="Edge" and e.valid?
    			edges.push(e)
  		end#if
	end#for
  	
  	
  	### Dialog

	@axis = "Z" if not @axis
	@base = "Center" if not @base
	@customvalue = 0.mm if not defined? @customvalue  and model.options["UnitsOptions"]["LengthUnit"]>1
  	@groupbase = "Center" if not @groupbase

	enums = ["X" '|' "Y" '|' "Z", "Max" '|' "Center" '|' "Min" '|' "Custom", "", "Max" '|' "Center" '|' "Min"]
	prompts = ["Align axis: ", "Align to: ", "Custom value: ", "Align group's: "]
	values = [@axis, @base, @customvalue, @groupbase]
	results = inputbox (prompts, values, enums, "Align Settings")
	axis, base, customvalue, groupbase = results
	@axis, @base, @customvalue, @groupbase = axis, base, customvalue, groupbase
	
  	
  	### Align
  		
	selverts = Sketchup.active_model.selection.vertices; 

	bb = Geom::BoundingBox.new;
	Sketchup.active_model.selection.each {|e| bb.add(e.bounds); } 
			

	if (@base == "Max")
		basepoint = Geom::Point3d.new (bb.max);
	end#if
	if (@base == "Center")
		basepoint = Geom::Point3d.new (bb.center);
	end#if
	if (@base == "Min")
		basepoint = Geom::Point3d.new (bb.min);
	end#if
	if (@base == "Custom")
		basepoint = Geom::Point3d.new ([customvalue, customvalue, customvalue]);
	end#if
	
	distances = []; 
	groupstomove = [];
	groupdistances = [];

	# Aligning groups
	Sketchup.active_model.selection.each {|e|
	if (e.kind_of? Sketchup::Group) or (e.kind_of? Sketchup::ComponentInstance) 

		groupstomove.push e

		if (@base == "Max")
			point = Geom::Point3d.new (e.bounds.max);
		end#if
		if (@base == "Center")
			point = Geom::Point3d.new (e.bounds.center);
		end#if
		if (@base == "Min")
			point = Geom::Point3d.new (e.bounds.min);
		end#if
		if (@base == "Custom") #Align the group's: max, min or center. (Just for groups/components.)
			if (@groupbase == "Max")
				point = Geom::Point3d.new (e.bounds.max);
			end#if
			if (@groupbase == "Center")
				point = Geom::Point3d.new (e.bounds.center);
			end#if
			if (@groupbase == "Min")
				point = Geom::Point3d.new (e.bounds.min);
			end#if
		end#if

		
		if (@axis == "X")
			dist = Geom::Point3d.new [basepoint.x-point.x, 0, 0];
		end#if
		if (@axis == "Y")
			dist = Geom::Point3d.new [0, basepoint.y-point.y, 0];
		end#if
		if (@axis == "Z")
			dist = Geom::Point3d.new [0, 0, basepoint.z-point.z];
		end#if

		groupdistances.push dist;
		
	end #if 
	}
	Sketchup.active_model.active_entities.transform_by_vectors groupstomove, groupdistances;


	# Aligning edges
	selverts.each {|v| 
	#####
		# This should appear in points.each do |p|,
		# and @vector should already be defined.

		dist = v.distance x #v is the vertex being transformed, x is the top vertex of the transformation.
		e = 2.71828182846
		half_rad = radius/2
		pos_from_half = half_rad - dist

		#@vector.length = (1 / ( 1 + e**-pos_from_half))*half_rad # this is the important part.
		@vector.length = (half_rad / ( 1 + @e**-(5/ half_rad*pos_from_half)**1))
		#@entities.transform_entities(@vector,p)
	#####
		point = Geom::Point3d.new [v.position.x, v.position.y, v.position.z];
		
		
		if (@axis == "X")
			dist = Geom::Point3d.new [basepoint.x-point.x, 0, 0];
		end#if
		if (@axis == "Y")
			dist = Geom::Point3d.new [0, basepoint.y-point.y, 0];
		end#if
		if (@axis == "Z")
			dist = Geom::Point3d.new [0, 0, basepoint.z-point.z];
		end#if

		distances.push dist; 
	}
	Sketchup.active_model.active_entities.transform_by_vectors selverts, distances;

end #jsAlign

############################################################
if( not file_loaded?("jsAlign.rb") )
    UI.menu("Plugins").add_separator
    UI.menu("Plugins").add_item("JS Align") { jsAlign  }
end

file_loaded("jsAlign.rb")
